#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Multi-consumer multi-producer dispatching mechanism
Originally based on pydispatch (BSD) http://pypi.python.org/pypi/PyDispatcher/2.0.1
See license.txt for original license.
"""

from dispatcher import Signal, receiver